#define DBUS_API_SUBJECT_TO_CHANGE
#include <dbus/dbus.h>
#include <stdbool.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#define OBJECTS_NBR 200

DBusConnection* conn;
pthread_mutex_t mutexConnection;
char signalName[200];


/**
 * Listens for signals on the bus
 */
void receive(int id)
{
   DBusMessage* msg;
   DBusMessageIter args;
   DBusError err;
   int ret;
   double value;

   printf("Listening for signals %d\n",id);

   // initialise the errors
   dbus_error_init(&err);
   

   // add a rule for which messages we want to see
   char * filter[100];
   pthread_mutex_lock (&mutexConnection);
   sprintf(filter,"type='signal',interface='net.prx.test',path='/net/prx/test/%d'",id);
   dbus_bus_add_match(conn, filter, &err); // see signals from the given interface
   dbus_connection_flush(conn);
   pthread_mutex_unlock (&mutexConnection);

   if (dbus_error_is_set(&err)) { 
      fprintf(stderr, "Match Error (%s)\n", err.message);
      exit(1); 
   }
   printf("Match rule sent\n");

   // loop listening for signals being emmitted
   while (true) {

      // non blocking read of the next available message
      pthread_mutex_lock (&mutexConnection);
      dbus_connection_read_write(conn, 0);
      msg = dbus_connection_pop_message(conn);
      pthread_mutex_unlock (&mutexConnection);

      // loop again if we haven't read a message
      if (NULL == msg) { 
         usleep(250000);
         continue;
      }

      // check if the message is a signal from the correct interface and with the correct name
      if (dbus_message_is_signal(msg, "net.prx.test", signalName)) {
         
         // read the parameters
         if (!dbus_message_iter_init(msg, &args))
            fprintf(stderr, "Message Has No Parameters\n");
         else if (DBUS_TYPE_DOUBLE!= dbus_message_iter_get_arg_type(&args)) 
            fprintf(stderr, "Argument is not string!\n"); 
         else
            dbus_message_iter_get_basic(&args, &value);
         
          printf("Got Signal from /net/prx/test/%d with value %f\n",id, value);
      }

   }
      // free the message
      dbus_message_unref(msg);
}


/*
/**
 * Listens for signals on the bus
 */
int connect_bus()
{
   DBusError err;
   int ret;

//   initialise the errors
   dbus_error_init(&err);
   
   // connect to the bus and check for errors
    conn = dbus_bus_get(DBUS_BUS_SYSTEM, &err);
   if (dbus_error_is_set(&err)) { 
      fprintf(stderr, "Connection Error (%s)\n", err.message);
      dbus_error_free(&err); 
   }
   if (NULL == conn) {
     printf ("Conn vale null\n");
     return 1;
   }
   printf ("Connected!\n");
   return 0;
}


int main(int argc, char** argv)
{

    int ret;
    int i=0;
    
 if (2 > argc) {
      printf ("Syntax: dbus-test-client <signal_name>\n");
      return 1;
   }

    sprintf (signalName,argv[1]);
      
    pthread_t threads[OBJECTS_NBR];
      if (connect_bus()==0){
        for (i=0;i<OBJECTS_NBR;i++)
        {
  	    ret = pthread_create(&threads[i], NULL, receive,i);
  	    if(ret) {
  		printf ("Error creating thread\n");
  		exit(1);
  	    }	
  	    pthread_detach( threads[i]);
  	}
 	while (1){
 	  usleep(500);
 	}
 	 dbus_connection_close(conn);
      }

   return 0;
}
