#define DBUS_API_SUBJECT_TO_CHANGE
#include <dbus/dbus.h>
#include <stdbool.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#define OBJECTS_NBR 200

DBusConnection* conn;
pthread_mutex_t mutexConnection;


/**
 * Connect to the DBUS bus and send a broadcast signal
 */
void send_signals_A(int id)
{
   DBusMessage* msg;
   DBusMessageIter args;
   DBusError err;
   int ret;
   dbus_uint32_t serial = 0;
  printf ("Starting to send singals A from %d\n",id);
  // add a rule for which messages we want to see
  char object_path[500];
  sprintf (object_path,"/net/prx/test/%d",id);
  double value=0;
  while (1){
   msg = dbus_message_new_signal(object_path, // object name of the signal
                                 "net.prx.test", // interface name of the signal
                                 "newSignalA"); // name of the signal
   if (NULL == msg) 
   { 
      fprintf(stderr, "Message Null\n"); 
      exit(1); 
   }
   
    // append arguments onto signal
    dbus_message_iter_init_append(msg, &args);
    if (!dbus_message_iter_append_basic(&args, DBUS_TYPE_DOUBLE, &value)) {
	fprintf(stderr, "Out Of Memory!\n"); 
	exit(1);
    }
    pthread_mutex_lock (&mutexConnection);
    // send the message and flush the connection
    if (!dbus_connection_send(conn, msg, &serial)) {
	fprintf(stderr, "Out Of Memory!\n"); 
	exit(1);
      }
      //dbus_connection_flush(conn);
      
      pthread_mutex_unlock (&mutexConnection);
      value ++;
      usleep (250000);
   }
      // free the message 
      dbus_message_unref(msg);            


   
}


/**
 * Connect to the DBUS bus and send a broadcast signal
 */
void send_signals_B(int id)
{
   DBusMessage* msg;
   DBusMessageIter args;
   DBusError err;
   int ret;
   dbus_uint32_t serial = 0;
  printf ("Starting to send singals B from %d\n",id);
  // add a rule for which messages we want to see
  char object_path[500];
  sprintf (object_path,"/net/prx/test/%d",id);
  double value=0;
  while (1){
   msg = dbus_message_new_signal(object_path, // object name of the signal
                                 "net.prx.test", // interface name of the signal
                                 "newSignalB"); // name of the signal
   if (NULL == msg) 
   { 
      fprintf(stderr, "Message Null\n"); 
      exit(1); 
   }
    // append arguments onto signal
    dbus_message_iter_init_append(msg, &args);
    if (!dbus_message_iter_append_basic(&args, DBUS_TYPE_DOUBLE, &value)) {
	fprintf(stderr, "Out Of Memory!\n"); 
	exit(1);
    }
    pthread_mutex_lock (&mutexConnection);
    // send the message and flush the connection
    if (!dbus_connection_send(conn, msg, &serial)) {
	fprintf(stderr, "Out Of Memory!\n"); 
	exit(1);
    }
      dbus_connection_flush(conn);
      
      pthread_mutex_unlock (&mutexConnection);
      value ++;     
      usleep (2000000);
      
      
   }
    // free the message 
    //dbus_message_unref(msg);

}


/*
/**
 * Listens for signals on the bus
 */
int connect_bus()
{
   DBusError err;
   int ret;

//   initialise the errors
   dbus_error_init(&err);
   
   // connect to the bus and check for errors
    conn = dbus_bus_get(DBUS_BUS_SYSTEM, &err);
   if (dbus_error_is_set(&err)) { 
      fprintf(stderr, "Connection Error (%s)\n", err.message);
      dbus_error_free(&err); 
   }
   if (NULL == conn) {
     printf ("Conn vale null\n");
     return 1;
   }
   
   // request our name on the bus and check for errors
   ret = dbus_bus_request_name(conn, "net.prx.genio.prx", DBUS_NAME_FLAG_REPLACE_EXISTING , &err);
   if (dbus_error_is_set(&err)) { 
      fprintf(stderr, "Name Error (%s)\n", err.message);
      dbus_error_free(&err); 
   }
   if (DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER != ret) {
     printf ("DBUS_REQUEST_NAME_REPLY_PRIMARY_OWNER != ret\n");
      return 1;
   }
   //===========================
   
   //===========================
   printf ("Connected!\n");
   return 0;
}


int main(int argc, char** argv)
{

    int ret;
    int i=0;
    

     pthread_t threads[OBJECTS_NBR*2];
       if (connect_bus()==0){
        for (i=0;i<OBJECTS_NBR*2;i++)
        {
  	    ret = pthread_create(&threads[i], NULL, send_signals_A,i);
  	    if(ret) {
  		printf ("Error creating thread\n");
  		exit(1);
  	    }	
 	    pthread_detach( threads[i]);
	    i++;
	    ret = pthread_create(&threads[i], NULL, send_signals_B,i);
 	    if(ret) {
 		printf ("Error creating thread\n");
 		exit(1);
 	    }	
 	    pthread_detach( threads[i]);
 	}
	while (1){
	  usleep(250000);
 	}
 	 dbus_connection_close(conn);
       }

   return 0;
}
